@@ -1,4 +1,4 @@
-/* $Id: child.c,v 1.142 2007/10/02 09:24:21 nicm Exp $ */
+/* $Id: child.c,v 1.146 2008/09/07 21:51:41 nicm Exp $ */
 
 /*
  * Copyright (c) 2006 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -29,6 +29,12 @@ void
 child_sighandler(int sig)
 {
 	switch (sig) {
+#ifdef SIGINFO
+	case SIGINFO:
+#endif
+	case SIGUSR1:
+		sigusr1 = 1;
+		break;
 	case SIGTERM:
 		cleanup_purge();
 		_exit(1);
@@ -48,6 +54,10 @@ child_fork(void)
 		cleanup_flush();
 
 		sigemptyset(&act.sa_mask);
+#ifdef SIGINFO
+		sigaddset(&act.sa_mask, SIGINFO);
+#endif
+		sigaddset(&act.sa_mask, SIGUSR1);
 		sigaddset(&act.sa_mask, SIGINT);
 		sigaddset(&act.sa_mask, SIGTERM);
 		act.sa_flags = SA_RESTART;
@@ -57,6 +67,12 @@ child_fork(void)
 			fatal("sigaction failed");
 
 		act.sa_handler = child_sighandler;
+#ifdef SIGINFO
+		if (sigaction(SIGINFO, &act, NULL) < 0)
+			fatal("sigaction failed");
+#endif
+		if (sigaction(SIGUSR1, &act, NULL) < 0)
+			fatal("sigaction failed");		
 		if (sigaction(SIGTERM, &act, NULL) < 0)
 			fatal("sigaction failed");
 
@@ -74,9 +90,10 @@ child_exit(int status)
 }
 
 struct child *
-child_start(struct children *children, uid_t uid, int (*start)(struct child *,
-    struct io *), int (*msg)(struct child *, struct msg *, struct msgbuf *),
-    void *data)
+child_start(struct children *children, uid_t uid, gid_t gid, 
+    int (*start)(struct child *, struct io *), 
+    int (*msg)(struct child *, struct msg *, struct msgbuf *),
+    void *data, struct child *parent)
 {
 	struct child	*child, *childp;
 	int		 fds[2], n;
@@ -90,6 +107,7 @@ child_start(struct children *children, uid_t uid, int (*start)(struct child *,
 	child->io = io_create(fds[0], NULL, IO_CRLF);
 	child->data = data;
 	child->msg = msg;
+	child->parent = parent;
 
 	if ((child->pid = child_fork()) == 0) {
 		for (i = 0; i < ARRAY_LENGTH(children); i++) {
@@ -101,7 +119,7 @@ child_start(struct children *children, uid_t uid, int (*start)(struct child *,
 		io_free(child->io);
 
 		if (geteuid() == 0)
-			dropto(uid);
+			dropto(uid, gid);
 
 		io = io_create(fds[1], NULL, IO_LF);
 		n = start(child, io);
