@@ -1,4 +1,4 @@
-/* $Id: connect.c,v 1.74 2007/12/03 20:53:18 nicm Exp $ */
+/* $Id: connect.c,v 1.76 2008/08/07 19:46:26 nicm Exp $ */
 
 /*
  * Copyright (c) 2006 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -37,11 +37,9 @@
 #include "fdm.h"
 
 int	sslverify(struct server *, SSL *, char **);
-#ifndef	NO_PROXY
 int	getport(char *);
 int	httpproxy(struct server *, struct proxy *, struct io *, int, char **);
 int	socks5proxy(struct server *, struct proxy *, struct io *, int, char **);
-#endif
 SSL    *makessl(struct server *, int, int, int, char **);
 
 char *
@@ -177,7 +175,6 @@ getaddrs(const char *host, char **fqdn, char **addr)
 	freeaddrinfo(ai);
 }
 
-#ifndef NO_PROXY
 struct proxy *
 getproxy(const char *xurl)
 {
@@ -528,7 +525,6 @@ httpproxy(struct server *srv,
 		xfree(line);
 	}
 }
-#endif /* NO_PROXY */
 
 SSL *
 makessl(struct server *srv, int fd, int verify, int timeout, char **cause)
@@ -620,6 +616,7 @@ connectio(
 	}
 
 	for (ai = srv->ai; ai != NULL; ai = ai->ai_next) {
+	retry:
 		fd = socket(ai->ai_family, ai->ai_socktype, ai->ai_protocol);
 		if (fd < 0) {
 			fn = "socket";
@@ -629,6 +626,8 @@ connectio(
 			error = errno;
 			close(fd);
 			errno = error;
+			if (errno == EINTR)
+				goto retry;
 			fd = -1;
 			fn = "connect";
 			continue;
