@@ -1,4 +1,4 @@
-/* $Id: deliver-drop.c,v 1.13 2006/12/11 13:07:08 nicm Exp $ */
+/* $Id: deliver-drop.c,v 1.18 2007/03/28 19:59:57 nicm Exp $ */
 
 /*
  * Copyright (c) 2006 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -18,23 +18,33 @@
 
 #include <sys/types.h>
 
+#include <string.h>
+
 #include "fdm.h"
+#include "deliver.h"
 
-int	 drop_deliver(struct deliver_ctx *, struct action *);
-char	*drop_desc(struct action *);
+int	 deliver_drop_deliver(struct deliver_ctx *, struct actitem *);
+void	 deliver_drop_desc(struct actitem *, char *, size_t);
 
-struct deliver deliver_drop = { DELIVER_INCHILD, drop_deliver, drop_desc };
+struct deliver deliver_drop = {
+	"drop",
+	DELIVER_INCHILD,
+	deliver_drop_deliver,
+	deliver_drop_desc
+};
 
 int
-drop_deliver(struct deliver_ctx *dctx, unused struct action *t)
+deliver_drop_deliver(struct deliver_ctx *dctx, unused struct actitem *ti)
 {
-	*dctx->decision = DECISION_DROP;
+	struct mail	*m = dctx->mail;
+
+	m->decision = DECISION_DROP;
 
 	return (DELIVER_SUCCESS);
 }
 
-char *
-drop_desc(unused struct action *t)
+void
+deliver_drop_desc(unused struct actitem *ti, char *buf, size_t len)
 {
-	return (xstrdup("drop"));
+	strlcpy(buf, "drop", len);
 }
