@@ -1,4 +1,4 @@
-/* $Id: fdm.c,v 1.169 2008/02/10 06:56:28 nicm Exp $ */
+/* $Id: fdm.c,v 1.180 2008/12/17 22:38:02 nicm Exp $ */
 
 /*
  * Copyright (c) 2006 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -43,9 +43,11 @@ const char		*malloc_options = "AFGJPRX";
 #endif
 
 void			 sighandler(int);
+struct child		*check_children(struct children *, u_int *);
 
 struct conf		 conf;
 
+volatile sig_atomic_t	 sigusr1;
 volatile sig_atomic_t	 sigint;
 volatile sig_atomic_t	 sigterm;
 
@@ -53,6 +55,12 @@ void
 sighandler(int sig)
 {
 	switch (sig) {
+#ifdef SIGINFO
+	case SIGINFO:
+#endif
+	case SIGUSR1:
+		sigusr1 = 1;
+		break;
 	case SIGINT:
 		sigint = 1;
 		break;
@@ -73,77 +81,23 @@ get_time(void)
 }
 
 void
-fill_info(const char *home)
+fill_host(void)
 {
-	struct passwd	*pw;
-	uid_t		 uid;
-	char		 host[MAXHOSTNAMELEN];
+	char	host[MAXHOSTNAMELEN];
 
-	uid = geteuid();
-	if (conf.info.valid && conf.info.last_uid == uid)
-		return;
-	conf.info.valid = 1;
-	conf.info.last_uid = uid;
-
-	if (conf.info.uid != NULL) {
-		xfree(conf.info.uid);
-		conf.info.uid = NULL;
-	}
-	if (conf.info.user != NULL) {
-		xfree(conf.info.user);
-		conf.info.user = NULL;
-	}
-	if (conf.info.home != NULL) {
-		xfree(conf.info.home);
-		conf.info.home = NULL;
-	}
-
-	if (conf.info.host == NULL) {
-		if (gethostname(host, sizeof host) != 0)
-			fatal("gethostname failed");
-		conf.info.host = xstrdup(host);
-
-		getaddrs(host, &conf.info.fqdn, &conf.info.addr);
-	}
-
-	if (home != NULL && *home != '\0')
-		conf.info.home = xstrdup(home);
-
-	xasprintf(&conf.info.uid, "%lu", (u_long) uid);
-	pw = getpwuid(uid);
-	if (pw != NULL) {
-		if (conf.info.home == NULL) {
-			if (pw->pw_dir != NULL && *pw->pw_dir != '\0')
-				conf.info.home = xstrdup(pw->pw_dir);
-			else
-				conf.info.home = xstrdup(".");
-		}
-		if (pw->pw_name != NULL && *pw->pw_name != '\0')
-			conf.info.user = xstrdup(pw->pw_name);
-	}
-	endpwent();
-	if (conf.info.user == NULL) {
-		conf.info.user = xstrdup(conf.info.uid);
-		log_warnx("can't find name for user %lu", (u_long) uid);
-	}
+	if (gethostname(host, sizeof host) != 0)
+		fatal("gethostname failed");
+	conf.host_name = xstrdup(host);
+	getaddrs(host, &conf.host_fqdn, &conf.host_address);
 }
 
 void
-dropto(uid_t uid)
+dropto(uid_t uid, gid_t gid)
 {
-	struct passwd	*pw;
-	gid_t		 gid;
-
 	if (uid == (uid_t) -1 || uid == 0)
 		return;
-
-	pw = getpwuid(uid);
-	if (pw == NULL) {
-		errno = ESRCH;
-		fatal("getpwuid failed");
-	}
-	gid = pw->pw_gid;
-	endpwent();
+	if (gid == (gid_t) -1 || gid == 0)
+		return;
 
 	if (setgroups(1, &gid) != 0)
 		fatal("setgroups failed");
@@ -213,11 +167,25 @@ use_account(struct account *a, char **cause)
 	return (1);
 }
 
+/* Check each child for a privsep message. */
+struct child *
+check_children(struct children *children, u_int *idx)
+{
+	struct child	*child;
+
+	for (*idx = 0; *idx < ARRAY_LENGTH(children); (*idx)++) {
+		child = ARRAY_ITEM(children, *idx);
+		if (child->io != NULL && privsep_check(child->io))
+			return (child);
+	}
+	return (NULL);
+}	
+
 __dead void
 usage(void)
 {
 	fprintf(stderr,
-	    "usage: %s [-klmnqv] [-a name] [-D name=value] [-f conffile] "
+	    "usage: %s [-hklmnqv] [-a name] [-D name=value] [-f conffile] "
 	    "[-u user] [-x name] [fetch|poll|cache] [arguments]\n", __progname);
         exit(1);
 }
@@ -228,17 +196,18 @@ main(int argc, char **argv)
         int		 opt, lockfd, status, res;
 	u_int		 i;
 	enum fdmop       op = FDMOP_NONE;
-	const char	*errstr, *proxy = NULL, *s;
-	char		 tmp[BUFSIZ], *ptr, *strs, *user = NULL, *lock = NULL;
+	const char	*proxy = NULL, *s;
+	char		 tmp[BUFSIZ], *ptr, *lock = NULL, *user, *home = NULL;
 	long		 n;
 	struct utsname	 un;
 	struct passwd	*pw;
 	struct stat	 sb;
 	time_t		 tt;
 	struct account	*a;
+	TAILQ_HEAD(, account) actaq; /* active accounts */
 	pid_t		 pid;
 	struct children	 children, dead_children;
-	struct child	*child;
+	struct child	*child, *child2;
 	struct io       *rio;
 	struct iolist	 iol;
 	double		 tim;
@@ -248,13 +217,14 @@ main(int argc, char **argv)
 	size_t		 off;
 	struct strings	 macros;
 	struct child_fetch_data *cfd;
+	struct userdata *ud;
 #ifdef DEBUG
 	struct rule	*r;
 	struct action	*t;
 	struct cache	*cache;
 #endif
 
-	log_open(stderr, LOG_MAIL, 0);
+	log_open_tty(0);
 
 	memset(&conf, 0, sizeof conf);
 	TAILQ_INIT(&conf.accounts);
@@ -270,15 +240,20 @@ main(int argc, char **argv)
 	conf.file_group = -1;
 	conf.queue_high = -1;
 	conf.queue_low = -1;
-	conf.def_user = -1;
-	conf.cmd_user = -1;
+	conf.def_user = NULL;
+	conf.cmd_user = NULL;
+	conf.max_accts = -1;
 	conf.strip_chars = xstrdup(DEFSTRIPCHARS);
 
+	conf.user_order = xmalloc(sizeof *conf.user_order);
+	ARRAY_INIT(conf.user_order);
+	ARRAY_ADD(conf.user_order, passwd_lookup);
+
 	ARRAY_INIT(&conf.incl);
 	ARRAY_INIT(&conf.excl);
 
 	ARRAY_INIT(&macros);
-        while ((opt = getopt(argc, argv, "a:D:f:klmnqu:vx:")) != EOF) {
+        while ((opt = getopt(argc, argv, "a:D:f:hklmnqu:vx:")) != -1) {
                 switch (opt) {
 		case 'a':
 			ARRAY_ADD(&conf.incl, xstrdup(optarg));
@@ -287,8 +262,12 @@ main(int argc, char **argv)
 			ARRAY_ADD(&macros, optarg);
 			break;
                 case 'f':
-                        conf.conf_file = xstrdup(optarg);
+			if (conf.conf_file == NULL)
+				conf.conf_file = xstrdup(optarg);
                         break;
+		case 'h':
+			home = getenv("HOME");
+			break;
 		case 'k':
 			conf.keep_all = 1;
 			break;
@@ -302,7 +281,8 @@ main(int argc, char **argv)
 			conf.check_only = 1;
 			break;
 		case 'u':
-			user = optarg;
+			if (conf.def_user == NULL)
+				conf.def_user = xstrdup(optarg);
 			break;
                 case 'v':
 			if (conf.debug != -1)
@@ -340,33 +320,11 @@ main(int argc, char **argv)
 			usage();
 	}
 
-	/* Check the user. */
-	if (user != NULL) {
-		pw = getpwnam(user);
-		if (pw == NULL) {
-			endpwent();
-			n = strtonum(user, 0, UID_MAX, &errstr);
-			if (errstr != NULL) {
-				if (errno == ERANGE) {
-					log_warnx("invalid uid: %s", user);
-					exit(1);
-				}
-			} else
-				pw = getpwuid((uid_t) n);
-			if (pw == NULL) {
-				log_warnx("unknown user: %s", user);
-				exit(1);
-			}
-		}
-		conf.def_user = pw->pw_uid;
-		endpwent();
-	}
-
 	/* Set debug level and start logging to syslog if necessary. */
 	if (conf.syslog)
-		log_open(NULL, LOG_MAIL, conf.debug);
+		log_open_syslog(conf.debug);
 	else
-		log_open(stderr, LOG_MAIL, conf.debug);
+		log_open_tty(conf.debug);
 	tt = time(NULL);
 	log_debug("version is: %s " BUILD ", started at: %.24s", __progname,
 	    ctime(&tt));
@@ -378,14 +336,28 @@ main(int argc, char **argv)
 	} else
 		log_debug2("uname: %s", strerror(errno));
 
-	/* Save the home dir and misc user info. */
-	fill_info(getenv("HOME"));
-	log_debug2("user is: %s, home is: %s", conf.info.user, conf.info.home);
+	/* Fill the hostname. */
+	fill_host();
+	log_debug2("host is: %s %s %s",
+	    conf.host_name, conf.host_fqdn, conf.host_address);
+
+	/* Find invoking user's details. */
+	if ((pw = getpwuid(getuid())) == NULL) {
+		log_warnx("unknown user: %lu", (u_long) geteuid());
+		exit(1); 
+	}
+ 	user = xstrdup(pw->pw_name);
+	if (home != NULL && *home != '\0')
+		conf.user_home = xstrdup(home);
+	else
+		conf.user_home = xstrdup(pw->pw_dir);
+	log_debug2("home is: %s", conf.user_home);
+	endpwent();
 
 	/* Find the config file. */
 	if (conf.conf_file == NULL) {
 		/* If no file specified, try ~ then /etc. */
-		xasprintf(&conf.conf_file, "%s/%s", conf.info.home, CONFFILE);
+		xasprintf(&conf.conf_file, "%s/%s", conf.user_home, CONFFILE);
 		if (access(conf.conf_file, R_OK) != 0) {
 			xfree(conf.conf_file);
 			conf.conf_file = xstrdup(SYSCONFFILE);
@@ -405,6 +377,13 @@ main(int argc, char **argv)
 	ARRAY_FREE(&macros);
 	log_debug2("configuration loaded");
 
+	/* Fill in users if not set already in configuration file. */
+	if (conf.def_user == NULL)
+		conf.def_user = xstrdup(user);
+	if (conf.cmd_user == NULL)
+		conf.cmd_user = xstrdup(user);
+	xfree(user);
+
 	/* Sort out queue limits. */
 	if (conf.queue_high == -1)
 		conf.queue_high = DEFMAILQUEUE;
@@ -417,16 +396,23 @@ main(int argc, char **argv)
 	/* Set the umask. */
 	umask(conf.file_umask);
 
-	/* Figure out default and command users. */
-	if (conf.def_user == (uid_t) -1) {
-		conf.def_user = geteuid();
-		if (conf.def_user == 0) {
-			log_warnx("no default user specified");
-			exit(1);
+	/* Check default and command users. */
+	if (conf.def_user == NULL) {
+		ud = user_lookup(conf.def_user, conf.user_order);
+		if (ud == NULL) {
+			log_warnx("unknown user: %s", conf.def_user);
+			exit(1); 
 		}
+		user_free(ud);
+	}
+	if (conf.cmd_user == NULL) {
+		ud = user_lookup(conf.cmd_user, conf.user_order);
+		if (ud == NULL) {
+			log_warnx("unknown user: %s", conf.cmd_user);
+			exit(1); 
+		}
+		user_free(ud);
 	}
-	if (conf.cmd_user == (uid_t) -1)
-		conf.cmd_user = geteuid();
 
 	/* Print proxy info. */
 	if (conf.proxy != NULL) {
@@ -459,33 +445,6 @@ main(int argc, char **argv)
 	}
 	log_debug2("locking using: %s", tmp);
 
-	/* Initialise and print headers and domains. */
-	if (conf.headers == NULL) {
-		conf.headers = xmalloc(sizeof *conf.headers);
-		ARRAY_INIT(conf.headers);
-		ARRAY_ADD(conf.headers, xstrdup("to"));
-		ARRAY_ADD(conf.headers, xstrdup("cc"));
-	}
-	strs = fmt_strings("", conf.headers);
-	log_debug2("headers are: %s", strs);
-	xfree(strs);
-	if (conf.domains == NULL) {
-		conf.domains = xmalloc(sizeof *conf.domains);
-		ARRAY_INIT(conf.domains);
-		ARRAY_ADD(conf.domains, xstrdup(conf.info.host));
-		if (conf.info.fqdn != NULL) {
-			ptr = xstrdup(conf.info.fqdn);
-			ARRAY_ADD(conf.domains, ptr);
-		}
-		if (conf.info.addr != NULL) {
-			xasprintf(&ptr, "\\[%s\\]", conf.info.addr);
-			ARRAY_ADD(conf.domains, ptr);
-		}
-	}
-	strs = fmt_strings("", conf.domains);
-	log_debug2("domains are: %s", strs);
-	xfree(strs);
-
 	/* Print the other settings. */
 	*tmp = '\0';
 	off = 0;
@@ -513,11 +472,11 @@ main(int argc, char **argv)
 	}
 	if (sizeof tmp > off) {
 		off += xsnprintf(tmp + off, (sizeof tmp) - off,
-		    "default-user=%lu, ", (u_long) conf.def_user);
+		    "default-user=\"%s\", ", conf.def_user);
 	}
 	if (sizeof tmp > off) {
 		off += xsnprintf(tmp + off, (sizeof tmp) - off,
-		    "command-user=%lu, ", (u_long) conf.cmd_user);
+		    "command-user=\"%s\", ", conf.cmd_user);
 	}
 	if (sizeof tmp > off && conf.impl_act != DECISION_NONE) {
 		if (conf.impl_act == DECISION_DROP)
@@ -542,6 +501,10 @@ main(int argc, char **argv)
 		    "queue-high=%u, queue-low=%u, ", conf.queue_high,
 		    conf.queue_low);
 	}
+	if (sizeof tmp > off && conf.max_accts != -1) {
+		off += xsnprintf(tmp + off, (sizeof tmp) - off,
+		    "parallel-accounts=%d ", conf.max_accts);
+	}
 	if (sizeof tmp > off && conf.lock_file != NULL) {
 		off += xsnprintf(tmp + off, (sizeof tmp) - off,
 		    "lock-file=\"%s\", ", conf.lock_file);
@@ -607,6 +570,10 @@ main(int argc, char **argv)
 	/* Set up signal handlers. */
 	memset(&act, 0, sizeof act);
 	sigemptyset(&act.sa_mask);
+#ifdef SIGINFO
+	sigaddset(&act.sa_mask, SIGINFO);
+#endif
+	sigaddset(&act.sa_mask, SIGUSR1);
 	sigaddset(&act.sa_mask, SIGINT);
 	sigaddset(&act.sa_mask, SIGTERM);
 	act.sa_flags = SA_RESTART;
@@ -620,6 +587,12 @@ main(int argc, char **argv)
 		fatal("sigaction failed");
 
 	act.sa_handler = sighandler;
+#ifdef SIGINFO
+	if (sigaction(SIGINFO, &act, NULL) < 0)
+		fatal("sigaction failed");
+#endif
+	if (sigaction(SIGUSR1, &act, NULL) < 0)
+		fatal("sigaction failed");
 	if (sigaction(SIGINT, &act, NULL) < 0)
 		fatal("sigaction failed");
 	if (sigaction(SIGTERM, &act, NULL) < 0)
@@ -631,7 +604,7 @@ main(int argc, char **argv)
 		if (geteuid() == 0)
 			lock = xstrdup(SYSLOCKFILE);
 		else
-			xasprintf(&lock, "%s/%s", conf.info.home, LOCKFILE);
+			xasprintf(&lock, "%s/%s", conf.user_home, LOCKFILE);
 	}
 	if (*lock != '\0' && !conf.allow_many) {
 		lockfd = xcreate(lock, O_WRONLY, -1, -1, S_IRUSR|S_IWUSR);
@@ -653,32 +626,22 @@ main(int argc, char **argv)
 	COUNTFDS("parent");
 #endif
 
-	/* Start the children and build the array. */
-	ARRAY_INIT(&children);
-	ARRAY_INIT(&dead_children);
-
-	child = NULL;
+	/* Filter account list. */
+	TAILQ_INIT(&actaq);
 	TAILQ_FOREACH(a, &conf.accounts, entry) {
-		if (!use_account(a, NULL))
-			continue;
-
-		cfd = xmalloc(sizeof *cfd);
-		cfd->account = a;
-		cfd->op = op;
-		cfd->children = &children;
-		child = child_start(&children, conf.child_uid, child_fetch,
-		    parent_fetch, cfd);
-
-		log_debug2("parent: child %ld (%s) started", (long) child->pid,
-		    a->name);
+		if (use_account(a, NULL))
+			TAILQ_INSERT_HEAD(&actaq, a, active_entry);
 	}
-
-	if (ARRAY_EMPTY(&children)) {
+	if (TAILQ_EMPTY(&actaq)) {
                 log_warnx("no accounts found");
 		res = 1;
 		goto out;
 	}
 
+	/* Initialise the child process arrays. */
+	ARRAY_INIT(&children);
+	ARRAY_INIT(&dead_children);
+
 #ifndef NO_SETPROCTITLE
 	setproctitle("parent");
 #endif
@@ -687,10 +650,31 @@ main(int argc, char **argv)
 
 	res = 0;
 	ARRAY_INIT(&iol);
-	while (!ARRAY_EMPTY(&children)) {
+	while (!TAILQ_EMPTY(&actaq) || ARRAY_LENGTH(&children) != 0) {
+		log_debug2("parent: %u children, %u dead children",
+		    ARRAY_LENGTH(&children), ARRAY_LENGTH(&dead_children));
+		
+		/* Stop on signal. */
 		if (sigint || sigterm)
 			break;
 
+		/* While there is space, start another child. */
+		while (!TAILQ_EMPTY(&actaq) && (conf.max_accts < 0 ||
+		    ARRAY_LENGTH(&children) < (u_int) conf.max_accts)) {
+			a = TAILQ_FIRST(&actaq);
+			TAILQ_REMOVE(&actaq, a, active_entry);
+			
+			cfd = xmalloc(sizeof *cfd);
+			cfd->account = a;
+			cfd->op = op;
+			cfd->children = &children;
+			child = child_start(&children, 
+			    conf.child_uid, conf.child_gid, 
+			    child_fetch, parent_fetch, cfd, NULL);
+			log_debug2("parent: child %ld (%s) started", 
+			    (long) child->pid, a->name);	
+		}
+
 		/* Fill the io list. */
 		ARRAY_CLEAR(&iol);
 		for (i = 0; i < ARRAY_LENGTH(&children); i++) {
@@ -707,32 +691,23 @@ main(int argc, char **argv)
 		case 0:
 			fatalx("child socket closed");
 		}
-
-		while (!ARRAY_EMPTY(&children)) {
-			/* Check all children for pending privsep messages. */
-			for (i = 0; i < ARRAY_LENGTH(&children); i++) {
-				child = ARRAY_ITEM(&children, i);
-				if (privsep_check(child->io))
-					break;
-			}
-			if (i == ARRAY_LENGTH(&children))
-				break;
-
-			/* And handle them if necessary. */
+		
+		/* Check all children for pending privsep messages. */
+		while ((child = check_children(&children, &i)) != NULL) {
+			/* Handle this message. */
 			if (privsep_recv(child->io, &msg, &msgbuf) != 0)
 				fatalx("privsep_recv error");
 			log_debug3("parent: got message type %d, id %u from "
 			    "child %ld", msg.type, msg.id, (long) child->pid);
-
 			if (child->msg(child, &msg, &msgbuf) == 0)
 				continue;
-
+		
 			/* Child has said it is ready to exit, tell it to. */
 			memset(&msg, 0, sizeof msg);
 			msg.type = MSG_EXIT;
 			if (privsep_send(child->io, &msg, NULL) != 0)
 				fatalx("privsep_send error");
-
+		
 			/* Wait for the child. */
 			if (waitpid(child->pid, &status, 0) == -1)
 				fatal("waitpid failed");
@@ -742,21 +717,35 @@ main(int argc, char **argv)
 				    (long) child->pid, WTERMSIG(status));
 			} else if (!WIFEXITED(status)) {
 				res = 1;
-				log_debug2("parent: child %ld didn't exit"
-				    "normally", (long) child->pid);
+				log_debug2("parent: child %ld exited badly",
+				    (long) child->pid);
 			} else {
 				if (WEXITSTATUS(status) != 0)
 					res = 1;
 				log_debug2("parent: child %ld returned %d",
 				    (long) child->pid, WEXITSTATUS(status));
 			}
-
+			
 			io_close(child->io);
 			io_free(child->io);
 			child->io = NULL;
-
+			
 			ARRAY_REMOVE(&children, i);
 			ARRAY_ADD(&dead_children, child);
+			
+			/*
+			 * If this child was the parent of any others, kill
+			 * them too.
+			 */
+			for (i = 0; i < ARRAY_LENGTH(&children); i++) {
+				child2 = ARRAY_ITEM(&children, i);
+				if (child2->parent != child)
+					continue;
+				
+				log_debug("parent: child %ld died: killing %ld",
+				    (long) child->pid, (long) child2->pid);
+				kill(child2->pid, SIGTERM);
+			}
 		}
 	}
 	ARRAY_FREE(&iol);
@@ -848,22 +837,19 @@ out:
 		TAILQ_REMOVE(&conf.actions, t, entry);
 		free_action(t);
 	}
-	xfree(conf.info.home);
-	xfree(conf.info.user);
-	xfree(conf.info.uid);
-	xfree(conf.info.host);
-	if (conf.info.fqdn != NULL)
-		xfree(conf.info.fqdn);
-	if (conf.info.addr != NULL)
-		xfree(conf.info.addr);
+	xfree(conf.def_user);
+	xfree(conf.cmd_user);
+	xfree(conf.user_home);
+	ARRAY_FREEALL(conf.user_order);
+	xfree(conf.host_name);
+	if (conf.host_fqdn != NULL)
+		xfree(conf.host_fqdn);
+	if (conf.host_address != NULL)
+		xfree(conf.host_address);
 	xfree(conf.conf_file);
 	xfree(conf.lock_file);
 	xfree(conf.tmp_dir);
 	xfree(conf.strip_chars);
-	free_strings(conf.domains);
-	ARRAY_FREEALL(conf.domains);
-	free_strings(conf.headers);
-	ARRAY_FREEALL(conf.headers);
 	free_strings(&conf.incl);
 	free_strings(&conf.excl);
 
