@@ -1,4 +1,4 @@
-/* $Id: fdm.h,v 1.335 2007/10/25 09:02:54 nicm Exp $ */
+/* $Id: fdm.h,v 1.344 2008/09/07 21:51:41 nicm Exp $ */
 
 /*
  * Copyright (c) 2006 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -67,6 +67,7 @@
 #define DEFUMASK	(S_IRWXG|S_IRWXO)
 #define FILEMODE	(S_IRUSR|S_IWUSR|S_IRGRP|S_IWGRP|S_IROTH|S_IWOTH)
 #define DIRMODE		(S_IRWXU|S_IRWXG|S_IRWXO)
+#define MAXUSERLEN	256
 
 extern char	*__progname;
 
@@ -138,9 +139,6 @@ extern char	*__progname;
 #define ENSURE_SIZE(buf, len, size) do {				\
 	(buf) = ensure_size(buf, &(len), 1, size);			\
 } while (0)
-#define ENSURE_SIZE2(buf, len, nmemb, size) do {			\
-	(buf) = ensure_size(buf, &(len), nmemb, size);			\
-} while (0)
 #define ENSURE_FOR(buf, len, size, adj) do {				\
 	(buf) = ensure_for(buf, &(len), size, adj);			\
 } while (0)
@@ -426,7 +424,8 @@ struct msgdata {
 	struct actitem			*actitem;
 	struct match_command_data	*cmddata;
 
-	uid_t			 	 uid;
+	uid_t				 uid;
+	gid_t	 			 gid;
 };
 
 /* Privsep message buffer. */
@@ -447,6 +446,7 @@ struct msg {
 /* A single child. */
 struct child {
 	pid_t		 pid;
+	struct child	*parent;
 	struct io	*io;
 
 	void		*data;
@@ -473,6 +473,9 @@ struct child_deliver_data {
 
 	struct child 		*child; /* the source of the request */
 
+	uid_t			 uid;
+	gid_t			 gid;
+
 	u_int			 msgid;
 	const char		*name;
 
@@ -486,17 +489,13 @@ struct child_deliver_data {
 	struct match_command_data *cmddata;
 };
 
-/* Users list. */
-ARRAY_DECL(users, uid_t);
-
 /* Account entry. */
 struct account {
 	u_int			 idx;
 
 	char			 name[MAXNAMESIZE];
 
-	struct users		*users;
-	int			 find_uid;
+	struct replstrs		*users;
 
 	int			 disabled;
 	int			 keep;
@@ -505,6 +504,7 @@ struct account {
 	void			*data;
 
 	TAILQ_ENTRY(account)	 entry;
+	TAILQ_ENTRY(account)	 active_entry;
 };
 
 /* Action item. */
@@ -524,8 +524,7 @@ TAILQ_HEAD(actlist, actitem);
 struct action {
 	char			 name[MAXNAMESIZE];
 
-	struct users		*users;
-	int			 find_uid;
+	struct replstrs		*users;
 
 	struct actlist		*list;
 
@@ -572,8 +571,7 @@ struct rule {
 
 	struct expr		*expr;
 
-	struct users		*users;
-	int			 find_uid;	/* find uids from headers */
+	struct replstrs		*users;
 
 	int			 stop;		/* stop matching at this rule */
 
@@ -589,6 +587,19 @@ struct rule {
 #define LOCK_FLOCK 0x2
 #define LOCK_DOTLOCK 0x4
 
+/* User info settings. */
+struct userdata {
+	char	*name;
+	char	*home;
+
+	uid_t	 uid;
+	gid_t	 gid;
+};
+
+/* User lookup order. */
+typedef struct userdata *(*userfunction)(const char *);
+ARRAY_DECL(userfunctions, userfunction);
+
 /* Configuration settings. */
 struct conf {
 	int 			 debug;
@@ -603,20 +614,12 @@ struct conf {
 
 	struct proxy		*proxy;
 
-	struct strings		*domains; /* domains to look for with users */
-	struct strings		*headers; /* headers to search for users */
+	char			*user_home;
+	struct userfunctions	*user_order;
 
-	struct {
-		int		 valid;
-		uid_t		 last_uid;
-
-		char		*home;
-		char		*user;
-		char		*uid;
-		char		*host;
-		char		*fqdn;
-		char		*addr;
-	} info;
+	char			*host_name;
+	char			*host_fqdn;
+	char			*host_address;
 
 	char			*conf_file;
 	char			*lock_file;
@@ -629,6 +632,7 @@ struct conf {
 	int			 verify_certs;
 	u_int			 purge_after;
 	enum decision		 impl_act;
+	int			 max_accts;
 
 	int			 queue_high;
 	int			 queue_low;
@@ -640,8 +644,9 @@ struct conf {
 	int			 timeout;
 	int		         del_big;
 	u_int			 lock_types;
-	uid_t			 def_user;
-	uid_t			 cmd_user;
+
+	char			*def_user;
+	char			*cmd_user;
 
 	TAILQ_HEAD(, cache)	 caches;
 	TAILQ_HEAD(, account)	 accounts;
@@ -729,11 +734,10 @@ int	 		parse_conf(const char *, struct strings *);
 __dead printflike1 void yyerror(const char *, ...);
 
 /* parse-fn.c */
-char		*expand_path(const char *);
+char		*expand_path(const char *, const char *);
 char		*run_command(const char *, const char *);
 char 		*fmt_replstrs(const char *, struct replstrs *);
 char 		*fmt_strings(const char *, struct strings *);
-char 		*fmt_users(const char *, struct users *);
 int		 have_accounts(char *);
 struct account	*find_account(char *);
 struct action  	*find_action(char *);
@@ -758,12 +762,15 @@ void		 netrc_close(FILE *);
 int		 netrc_lookup(FILE *, const char *, char **, char **);
 
 /* fdm.c */
+extern volatile sig_atomic_t sigusr1;
+extern volatile sig_atomic_t sigint;
+extern volatile sig_atomic_t sigterm;
 double		 get_time(void);
-void		 dropto(uid_t);
+void		 dropto(uid_t, gid_t);
 int		 check_incl(const char *);
 int	         check_excl(const char *);
 int		 use_account(struct account *, char **);
-void		 fill_info(const char *);
+void		 fill_host(void);
 __dead void	 usage(void);
 
 /* cache-op.c */
@@ -782,6 +789,19 @@ void printflike2 attach_log(struct attach *, const char *, ...);
 struct attach 	*attach_build(struct mail *);
 void		 attach_free(struct attach *);
 
+/* lookup.c */
+struct userdata *user_lookup(const char *, struct userfunctions *);
+void		 user_free(struct userdata *);
+struct userdata *user_copy(struct userdata *);
+
+/* lookup-passwd.c */
+struct userdata *passwd_lookup(const char *);
+
+#ifdef LOOKUP_COURIER
+/* lookup-courier.c */
+struct userdata *courier_lookup(const char *);
+#endif
+
 /* privsep.c */
 int		 privsep_send(struct io *, struct msg *, struct msgbuf *);
 int		 privsep_check(struct io *);
@@ -796,9 +816,10 @@ void		 cmd_free(struct cmd *);
 /* child.c */
 int		 child_fork(void);
 __dead void	 child_exit(int);
-struct child 	*child_start(struct children *, uid_t, int (*)(struct child *,
-    		     struct io *), int (*)(struct child *, struct msg *,
-    		     struct msgbuf *), void *);
+struct child 	*child_start(struct children *, uid_t, gid_t, 
+    		     int (*)(struct child *, struct io *),
+    		     int (*)(struct child *, struct msg *, struct msgbuf *),
+		     void *, struct child *);
 
 /* child-fetch.c */
 int		 open_cache(struct account *, struct cache *);
@@ -861,10 +882,9 @@ char		*match_header(struct mail *, const char *, size_t *, int);
 size_t		 find_body(struct mail *);
 void		 count_lines(struct mail *, u_int *, u_int *);
 int		 append_line(struct mail *, const char *, size_t);
-struct users	*find_users(struct mail *);
 char		*find_address(char *, size_t, size_t *);
 void		 trim_from(struct mail *);
-char 	        *make_from(struct mail *);
+char 	        *make_from(struct mail *, char *);
 u_int		 fill_wrapped(struct mail *);
 void		 set_wrapped(struct mail *, char);
 
@@ -910,17 +930,21 @@ void printflike3 add_tag(struct strb **, const char *, const char *, ...);
 const char	*find_tag(struct strb *, const char *);
 const char	*match_tag(struct strb *, const char *);
 void		 default_tags(struct strb **, const char *);
-void		 update_tags(struct strb **);
+void		 update_tags(struct strb **, struct userdata *);
+void		 reset_tags(struct strb **);
 char 		*replacestr(struct replstr *, struct strb *, struct mail *,
     		     struct rmlist *);
 char 		*replacepath(struct replpath *, struct strb *, struct mail *,
-   		     struct rmlist *);
+		     struct rmlist *, const char *);
 
 /* log.c */
-void		 log_open(FILE *, int, int);
+#define LOG_FACILITY LOG_MAIL
+void		 log_open_syslog(int);
+void		 log_open_tty(int);
+void		 log_open_file(int, const char *);
 void		 log_close(void);
-void		 log_vwrite(FILE *, int, const char *, va_list);
-void		 log_write(FILE *, int, const char *, ...);
+void		 log_vwrite(int, const char *, va_list);
+void		 log_write(int, const char *, ...);
 void printflike1 log_warn(const char *, ...);
 void printflike1 log_warnx(const char *, ...);
 void printflike1 log_info(const char *, ...);
@@ -934,7 +958,6 @@ __dead void	 log_fatalx(const char *, ...);
 /* xmalloc.c */
 void		*ensure_size(void *, size_t *, size_t, size_t);
 void		*ensure_for(void *, size_t *, size_t, size_t);
-char		*xmemstrdup(const char *, size_t);
 char		*xstrdup(const char *);
 void		*xcalloc(size_t, size_t);
 void		*xmalloc(size_t);
