@@ -1,4 +1,4 @@
-/* $Id: fetch.h,v 1.46 2007/09/19 09:05:40 nicm Exp $ */
+/* $Id: fetch.h,v 1.50 2008/10/28 07:01:39 nicm Exp $ */
 
 /*
  * Copyright (c) 2007 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -193,24 +193,23 @@ struct fetch_imap_data {
 
 	char		*user;
 	char		*pass;
-	char		*folder;
 	struct server	 server;
 	char		*pipecmd;
+	int		 nocrammd5;
+	int		 nologin;
+
+	u_int		 folder;
+	struct strings	*folders;
 
 	int		 capa;
 	int		 tag;
 
-	u_int		 cur;
-	u_int		 num;
+	ARRAY_DECL(, u_int) wanted;
+	ARRAY_DECL(, u_int) dropped;
 
 	u_int		 total;
 	u_int		 committed;
 
-	u_int	 	 uid;
-
-	ARRAY_DECL(, u_int) kept;
-	TAILQ_HEAD(, fetch_imap_mail) dropped;
-
 	int		 flushing;
 	size_t		 size;
 	u_int		 lines;
@@ -228,11 +227,17 @@ struct fetch_imap_data {
 
 struct fetch_imap_mail {
 	u_int		 uid;
-	u_int		 idx;
-
-	TAILQ_ENTRY(fetch_imap_mail) entry;
 };
 
+#define IMAP_TAG_NONE -1
+#define IMAP_TAG_CONTINUE -2
+#define IMAP_TAG_ERROR -3
+
+#define IMAP_TAGGED 0
+#define IMAP_CONTINUE 1
+#define IMAP_UNTAGGED 2
+#define IMAP_RAW 3
+
 /* fetch-maildir.c */
 extern struct fetch 	 fetch_maildir;
 
@@ -253,12 +258,24 @@ extern struct fetch 	 fetch_pop3pipe;
 
 /* fetch-imap.c */
 extern struct fetch 	 fetch_imap;
+int	fetch_imap_putln(struct account *, const char *, va_list);
+int	fetch_imap_getln(struct account *, struct fetch_ctx *, char **);
+int	fetch_imap_state_init(struct account *, struct fetch_ctx *);
 
 /* fetch-imappipe.c */
 extern struct fetch 	 fetch_imappipe;
 
 /* imap-common.c */
+int	imap_tag(char *);
+int	imap_putln(struct account *, const char *, ...);
+int	imap_getln(struct account *, struct fetch_ctx *, int, char **);
+int	imap_okay(char *);
+int	imap_no(char *);
+int	imap_bad(struct account *, const char *);
+int	imap_invalid(struct account *, const char *);
 int	imap_state_init(struct account *, struct fetch_ctx *);
+int	imap_state_connected(struct account *, struct fetch_ctx *);
+int	imap_state_select1(struct account *, struct fetch_ctx *);
 int	imap_commit(struct account *, struct mail *);
 void	imap_abort(struct account *);
 u_int	imap_total(struct account *);
