@@ -1,4 +1,4 @@
-/* $Id: privsep.c,v 1.9 2007/03/27 10:07:39 nicm Exp $ */
+/* $Id: privsep.c,v 1.11 2007/07/25 22:05:06 nicm Exp $ */
 
 /*
  * Copyright (c) 2006 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -30,13 +30,13 @@ privsep_send(struct io *io, struct msg *msg, struct msgbuf *msgbuf)
 		msg->size = msgbuf->len;
 
 	io_write(io, msg, sizeof *msg);
-	if (io_flush(io, &cause) != 0)
-		return (1);
+	if (io_flush(io, INFTIM, &cause) != 0)
+		return (-1);
 
 	if (msg->size != 0) {
 		io_write(io, msgbuf->buf, msgbuf->len);
-		if (io_flush(io, &cause) != 0)
-			return (1);
+		if (io_flush(io, INFTIM, &cause) != 0)
+			return (-1);
 	}
 
 	return (0);
@@ -56,21 +56,21 @@ privsep_recv(struct io *io, struct msg *msg, struct msgbuf *msgbuf)
 		msgbuf->len = 0;
 	}
 
-	if (io_wait(io, sizeof *msg, NULL) != 0)
-		return (1);
+	if (io_wait(io, sizeof *msg, INFTIM, NULL) != 0)
+		return (-1);
 	if (io_read2(io, msg, sizeof *msg) != 0)
-		return (1);
+		return (-1);
 
 	if (msg->size == 0)
 		return (0);
 
 	if (msgbuf == NULL)
-		return (1);
+		return (-1);
 	msgbuf->len = msg->size;
-	if (io_wait(io, msgbuf->len, NULL) != 0)
-		return (1);
+	if (io_wait(io, msgbuf->len, INFTIM, NULL) != 0)
+		return (-1);
 	if ((msgbuf->buf = io_read(io, msgbuf->len)) == NULL)
-		return (1);
+		return (-1);
 
 	return (0);
 }
